/*************************************************************************/
/*  Job Server  UNIX Version                                             */
/*                                                                       */
/*  Zeno Job Server                                                      */
/*  Copyright (C) 2005,2006  Scott Auge, Amduus Information Works, Inc.  */
/*                                                                       */
/*  This program is free software; you can redistribute it and/or modify */
/*  it under the terms of the GNU General Public License as published by */
/*  the Free Software Foundation; either version 2 of the License, or    */
/*  (at your option) any later version.                                  */
/*                                                                       */
/*  This program is distributed in the hope that it will be useful,      */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/*  GNU General Public License for more details.                         */
/*                                                                       */
/*  You should have received a copy of the GNU General Public License    */
/*  along with this program; if not, write to                            */
/*                                                                       */
/*  Free Software Foundation, Inc.                                       */
/*  59 Temple Place, Suite 330                                           */
/*  Boston, MA  02111-1307  USA                                          */
/*                                                                       */
/*************************************************************************/

/*************************************************************************/
/* Request a service be performed by a processor                         */
/*************************************************************************/

PROCEDURE SubmitJob:
  
  DEFINE INPUT PARAMETER cPrcName  AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER iPriority AS INTEGER NO-UNDO.
  DEFINE INPUT PARAMETER cDataIn   AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER cJobID   AS CHARACTER NO-UNDO.
  
  DEFINE BUFFER Job_NewJob FOR Job.
  
  /*********************************************************************/
  /* Be clear we want this transaction to end when we are done with it */
  /*********************************************************************/
      
  CREATE Job_NewJob.
  
  RUN makeid3.p (10, OUTPUT Job_NewJob.JobID).
  
  ASSIGN 
  Job_NewJob.State = "SUBMITTED"
  Job_NewJob.Name = cPrcName
  Job_NewJob.Priority = iPriority
  Job_NewJob.ErrCode = "000"
  Job_NewJob.SubmitDate = TODAY
  Job_NewJob.SubmitTime = TIME
  Job_NewJob.PrgDataIn = cDataIn.
  
  ASSIGN cJobID = Job_NewJob.JobID.
  
  /*********************************************************************/  
  /* Determine if there is such a service defined                      */
  /*********************************************************************/
  
  IF NOT CAN-FIND (FIRST JobPrg WHERE JobPrg.Name = cPrcName) THEN DO:
    
    ASSIGN 
    Job_NewJob.State = "ERROR"
    Job_NewJob.ErrCode = "100".
    
  END. /* IF NOT CAN-FIND () */
  
  RELEASE Job_NewJob.
  
END. /* PROCEDURE SubmitJob */

/*************************************************************************/
/* Watch the state of the job.                                           */
/*************************************************************************/

PROCEDURE WatchJob:
  
  DEFINE INPUT PARAMETER cJobID AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER cState AS CHARACTER NO-UNDO.
  
  DEFINE BUFFER Job_WatchJob FOR Job.
  
  FIND Job_WatchJob NO-LOCK
  WHERE Job_WatchJob.JobID = cJobID
  NO-ERROR.
  
  IF NOT AVAILABLE Job_WatchJob THEN ASSIGN cState = "NOT FOUND".
  ELSE ASSIGN cState = Job_WatchJob.State.

END. /* PROCEDURE WatchJob */

/*************************************************************************/
/* Cancel a job.                                                         */
/*************************************************************************/

PROCEDURE CancelJob:

  DEFINE INPUT PARAMETER cJobID AS CHARACTER NO-UNDO.

  DEFINE BUFFER Job_CancelJob FOR Job.
  
  FIND Job_CancelJob EXCLUSIVE-LOCK
  WHERE Job_CancelJob.JobID = cJobID.
  
  ASSIGN Job_CancelJob.State = "CANCELLED".
  
  RELEASE Job_CancelJob.
  
END. /* PROCEDURE CancelJob */

/*************************************************************************/
/* Return the Job buffer for the user to use.                            */
/* Returns the buffer in NO-LOCK status.                                 */
/*************************************************************************/

PROCEDURE GetJob:

  DEFINE INPUT PARAMETER cJobID AS CHARACTER NO-UNDO.
  DEFINE PARAMETER BUFFER Job_TheJob FOR Job.
  
  FIND Job_TheJob NO-LOCK
  WHERE Job_TheJob.JobID = cJobID
  NO-ERROR.
  
END. /* PROCEDURE JobResults */

/*************************************************************************/
/* Return the Job output parameter for the user to use.                  */
/*************************************************************************/

PROCEDURE GetJobResults:

  DEFINE INPUT PARAMETER cJobID AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER cResults AS CHARACTER NO-UNDO.

  DEFINE BUFFER Job_TheJob FOR Job.
  
  FIND Job_TheJob NO-LOCK
  WHERE Job_TheJob.JobID = cJobID
  NO-ERROR.
  
  ASSIGN cResults = Job_TheJob.PrgDataOut.
  
END. /* PROCEDURE JobResults */

/*************************************************************************/
/* Match a service name (JobCtrl.Name) to a progress program to do it.   */
/*************************************************************************/

PROCEDURE DefineService:

  DEFINE INPUT PARAMETER cServiceName AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER c4GLPrg AS CHARACTER NO-UNDO.
  
  FIND JobPrg EXCLUSIVE-LOCK 
  WHERE JobPrg.Name = cServiceName
  NO-ERROR.
  
  IF AVAILABLE JobPrg THEN DO:
    
    ASSIGN JobPrg.PrgName = c4GLPrg.
    RETURN.
    
  END. /* IF AVAILABLE ... TRANSACTION */

  CREATE JobPrg.
  
  RUN makeid3.p (INPUT 10, OUTPUT JobPrg.JobPrgID).
  
  ASSIGN 
  JobPrg.Name = cServiceName
  JobPrg.PrgName = c4GLPrg.
  
END. /* PROCEDURE DefineService */

/*************************************************************************/
/* Delete the given service from the system.  This does NOT delete any   */
/* source code!  We do a FOR EACH in case, somehow, there are more than  */
/* one JobPrg record for a given service (this means someone wasn't      */
/* using this API to work the table!)                                    */
/*************************************************************************/

PROCEDURE RemoveService:
  
  DEFINE INPUT PARAMETER cServiceName AS CHARACTER NO-UNDO.

  FOR EACH JobPrg EXCLUSIVE-LOCK
    WHERE JobPrg.Name = cServiceName:
  
    DELETE JobPrg.
    
  END. /* FOR EACH JobPrg */
  
END.

