/*************************************************************************/
/*  Job Server  UNIX Version                                             */
/*                                                                       */
/*  Zeno Job Server                                                      */
/*  Copyright (C) 2005  Scott Auge, Amduus Information Works, Inc.       */
/*                                                                       */
/*  This program is free software; you can redistribute it and/or modify */
/*  it under the terms of the GNU General Public License as published by */
/*  the Free Software Foundation; either version 2 of the License, or    */
/*  (at your option) any later version.                                  */
/*                                                                       */
/*  This program is distributed in the hope that it will be useful,      */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/*  GNU General Public License for more details.                         */
/*                                                                       */
/*  You should have received a copy of the GNU General Public License    */
/*  along with this program; if not, write to                            */
/*                                                                       */
/*  Free Software Foundation, Inc.                                       */
/*  59 Temple Place, Suite 330                                           */
/*  Boston, MA  02111-1307  USA                                          */
/*                                                                       */
/*************************************************************************/
                                       
DEFINE VARIABLE cHostPID        AS CHARACTER NO-UNDO.
DEFINE VARIABLE cHostName       AS CHARACTER NO-UNDO.                                                       
DEFINE VARIABLE cJobCtrlID      AS CHARACTER NO-UNDO.
DEFINE VARIABLE cName           AS CHARACTER NO-UNDO.
DEFINE VARIABLE cCmd            AS CHARACTER NO-UNDO.
DEFINE VARIABLE cToPrcStopDate  AS CHARACTER NO-UNDO.
DEFINE VARIABLE cToSubmitDate   AS CHARACTER NO-UNDO.
DEFINE VARIABLE dToPrcStopDate  AS DATE NO-UNDO.
DEFINE VARIABLE dToSubmitDate   AS DATE NO-UNDO.
DEFINE VARIABLE iPause          AS INTEGER NO-UNDO.
DEFINE VARIABLE cgThisHost      AS CHARACTER NO-UNDO.
DEFINE VARIABLE cgThisPID       AS CHARACTER NO-UNDO.

/*************************************************************************/
/* Interprete from the command line invocation what we are trying to do. */
/*************************************************************************/

/*************************************************************************/
/* Give a means for the process to talk back to the user on the command  */
/* line.  This can be redirected into a log file if so desired.          */
/*************************************************************************/

OUTPUT TO /dev/tty.

/**********************************************************************/
/* Don't get your hostnames and pids mixed up.  cHostName is for stop */
/* on potential other hosts.  The cgThisHost is for THIS host.        */
/**********************************************************************/

RUN gethostname.p (OUTPUT cgThisHost).
RUN gethostpid.p (OUTPUT cgThisPID).

PUT UNFORMATTED "This host is " cgThisHost SKIP.
PUT UNFORMATTED "This PID is " cgThisPID SKIP.
  
ASSIGN 
cHostPID       = OS-GETENV("PRCHOSTPID")
cHostName      = OS-GETENV("PRCHOSTNAME")
cName          = OS-GETENV("PRCNAME")
cCmd           = OS-GETENV("PRCCMD")
cToPrcStopDate = OS-GETENV("TOSTOPDATE")
cToSubmitDate  = OS-GETENV("TOSUBMITDATE")
iPause         = INT(OS-GETENV("PRCPAUSE"))
NO-ERROR.
                                                                                                       
IF iPause = ? THEN iPause = 0.

CASE cCmd:
  
  /***********************************************************************/
  /* There is no coming back from the start command.                     */
  /* Starting a process starts the process on the machine the command    */
  /* was invoked on.  One cannot mngprc.bash -start on one machine and   */
  /* have the process appear on another machine.                         */
  /***********************************************************************/
  
  WHEN "start" THEN DO:

    PUT UNFORMATTED "Starting the processor..." SKIP.
    RUN StartProcessor.

  END. /* WHEN start */

  /***********************************************************************/
  /* Since we use database records to communicate with job processes, we */
  /* can stop processes living on other machines with messages in the    */
  /* JobCtrl table.                                                      */
  /* Even if you need to kill the processor, you should issue this com-  */
  /* mand so the records are marked for clean up.                        */  
  /***********************************************************************/
  
  WHEN "stop" THEN DO:
   
    IF cHostName <> ? AND cHostPID <> ? THEN 
      RUN StopByHostAndPid (INPUT cHostName, INPUT cHostPID).
    
    ELSE IF cHostName <> ? AND cName <> ? THEN
      RUN StopByHostAndName (INPUT cHostName, INPUT cName).
    
    ELSE DO:
      
      PUT UNFORMATTED "You tried to stop, but didn't issue the right com-" SKIP
                      "bination of arguments.  They need to be:          " SKIP
                      "-stop -hostname -hostpid" SKIP
                      "OR" SKIP
                      "-stop -hostname -prcname"
                      SKIP.
                      
    END. /* Error out! */
    
    QUIT.
    
  END. /* WHEN stop */

  /***********************************************************************/
  /* After multiple stops and starts and processed records, cruft can    */
  /* build up in the table.  The following provides a manner of purging  */
  /* old records from the database table.  One should run a clean once a */
  /* week or so.                                                         */
  /* One can clean all records up to a SubmitDate or to a PrcStopDate.   */
  /* In case a record is never processed, one may want to do PrcStopDate */
  /* instead of a SubmitDate so you can see the stragglers.  If the time */
  /* has simply passed, do so by SubmitDate.                             */
  /***********************************************************************/
  
  WHEN "clean" THEN DO:
  
    PUT UNFORMATTED "Cleaning..." SKIP.
    
    RUN CleanJobCtrl.
        
    ASSIGN 
    dToSubmitDate = DATE(cToSubmitDate)
    dToPrcStopDate = DATE(cToPrcStopDate)
    NO-ERROR.
    
    RUN CleanJob (INPUT dToSubmitDate, INPUT dToPrcStopDate).
    
    QUIT.
    
  END. /* WHEN clean */
  /***********************************************************************/
  /* Explain our cluelessness to junk commands.                          */
  /***********************************************************************/
  
  OTHERWISE DO:
    PUT UNFORMATTED "I don't understand what you are asking!" SKIP.
  END. /* OTHERWISE */
  
END. /* CASE cCmd */

OUTPUT CLOSE.

/*************************************************************************/
/* Should the command be start up - then start up the processor.         */ 
/*************************************************************************/

PROCEDURE StartProcessor:
                     
  RUN Init.
  
  RUN SearchForWork.               
    
END. /* PROCEDURE StartProcessor */

/*************************************************************************/
/* Initialize the processor and populate it's record JobCtrl record.     */
/*************************************************************************/

PROCEDURE Init:

  DEFINE VARIABLE cInitHostName AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cInitHostPID  AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cJobCtrlID   AS CHARACTER NO-UNDO.

  /********************************************************************/ 
  /* Create a unique record identifier for an upcoming JobCtrl.       */
  /********************************************************************/ 

  RUN makeid3.p (INPUT 10, OUTPUT cJobCtrlID).
  
  /********************************************************************/ 
  /* Get the facts about this machine.                                */
  /********************************************************************/ 
  
  RUN gethostname.p (OUTPUT cInitHostName).
  RUN gethostpid.p (OUTPUT cInitHostPID).
  
  /********************************************************************/ 
  /* Prevent duplicate PID/Hostname records                           */
  /********************************************************************/
                               
  RUN CleanJobCtrl.         

  /********************************************************************/
  /* Create a JobCtrl record for this process.                        */
  /* DO TRANSACTION because we want to commit ASAP so the monitoring  */
  /* apps can find the record.                                        */
  /********************************************************************/
 
  CREATE JobCtrl.
  
  ASSIGN
  JobCtrl.PrcHostName = cInitHostName 
  JobCtrl.PrcHostPID  = cInitHostPID
  JobCtrl.PrcName     = cName
  JobCtrl.Operation   = "Running"
  JobCtrl.JobCtrlID   = cJobCtrlID 
  JobCtrl.ActionAlert = ""            
  JobCtrl.JobID       = "".
  
  RELEASE JobCtrl.
  
END. /* PROCEDURE Init */

/*************************************************************************/
/* Search for work - this is the meat of the job server                  */
/*************************************************************************/
                                      
PROCEDURE SearchForWork:
  
  /*********************************************************************/
  /* Loop through the Job records looking for something to do.  When   */
  /* we are done with a job, check the JobCtrl record for this process */
  /* to determine if we should stop.                                   */
  /*********************************************************************/
  
  REPEAT:

    RUN CheckJobCtrl.
    RUN CheckJob.
    
    PAUSE iPause.
    
  END.  /* REPEAT */
  
END. /* SearchForWork */

/*************************************************************************/
/* Shutdown - update the JobCtrl record and remove from the OS.          */
/* Use this to stop an individual process on the given host.             */
/*************************************************************************/

PROCEDURE StopByHostAndPid:

  DEFINE INPUT PARAMETER cHostName AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER cHostPID AS CHARACTER NO-UNDO.

  FOR EACH JobCtrl EXCLUSIVE-LOCK
    WHERE JobCtrl.PrcHostName = cHostName
      AND JobCtrl.PrcHostPID = cHostPID
      AND JobCtrl.Operation <> "Stopped":
      
      PUT UNFORMATTED "Stopping " JobCtrl.PrcHostPID
                      " on " JobCtrl.PrcHostName
                      SKIP.
                      
      ASSIGN JobCtrl.ActionAlert = "STOP".
      
  END. /* FOR EACH JobCtrl */
  
END. /* PROCEDURE StopByHostAndPid */

/*************************************************************************/
/* Shutdown - update the JobCtrl record and remove from the OS.          */
/* Use this to stop all processors by name on a given host.              */
/*************************************************************************/

PROCEDURE StopByHostAndName:

  DEFINE INPUT PARAMETER cHostName AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER cPrcName AS CHARACTER NO-UNDO.
  
  PUT UNFORMATTED "Stopping all processes named " cName " on " cHostName SKIP.
  
  FOR EACH JobCtrl EXCLUSIVE-LOCK
    WHERE JobCtrl.PrcHostName = cHostName
      AND JobCtrl.PrcName = cPrcName
      AND JobCtrl.Operation <> "Stopped":
      
      PUT UNFORMATTED "Stopping " JobCtrl.PrcHostPID
                      " on " JobCtrl.PrcHostName
                      SKIP.
                      
      ASSIGN JobCtrl.ActionAlert = "STOP".
      
  END. /* FOR EACH JobCtrl */

END. /* PROCEDURE StopByHostAndName */

/*************************************************************************/
/* Delete Job records up to a given date.                                */
/*************************************************************************/

PROCEDURE CleanJob:
  
  DEFINE INPUT PARAMETER dToSubmitDate AS DATE NO-UNDO.
  DEFINE INPUT PARAMETER dToPrcStopDate AS DATE NO-UNDO.
  
  DEFINE VARIABLE lDidSomeCleaning AS LOGICAL NO-UNDO.
  
  ASSIGN lDidSomeCleaning = NO.

  PUT UNFORMATTED "Looking for some Job records to delete..." SKIP.
  
  IF dToSubmitDate <> ? THEN DO:
  
    ASSIGN lDidSomeCleaning = YES.
  
    FOR EACH Job EXCLUSIVE-LOCK
       WHERE Job.SubmitDate <= dToSubmitDate:
  
      PUT UNFORMATTED "Deleting Job " Job.Name
                      " State " Job.State
                      " ErrCode " Job.ErrCode
                      " On Host " Job.PrcHostName
                      " For PID " Job.PrcHostPID
                      SKIP.
                      
      DELETE Job.
      
    END. /* FOR EACH Job */
    
  END. /* IF dToSubmitDate <> ? */
  
  IF dToPrcStopDate <> ? THEN DO:

    ASSIGN lDidSomeCleaning = YES.
  
    FOR EACH Job EXCLUSIVE-LOCK
      WHERE Job.State = "Stopped"
        AND Job.PrcStopDate <= dToPrcStopDate:
  
      PUT UNFORMATTED "Deleting Job " Job.Name
                      " State " Job.State
                      " ErrCode " Job.ErrCode
                      " On Host " Job.PrcHostName
                      " For PID " Job.PrcHostPID
                      SKIP.
                      
      DELETE Job.
      
    END. /* FOR EACH Job */
    
  END. /* IF dToPrcStopDate <> ? */ 
  
  IF NOT lDidSomeCleaning THEN DO:
    
    PUT UNFORMATTED "Looks like we were to do some cleaning, but the "
                    "dates were messed up and so no cleaning was done!"
                    SKIP.
                    
  END. /* IF NOT lDidSomeCleaning */
  
  PUT UNFORMATTED "Done looking for Job records to delete..." SKIP.
  
END. /* PROCEDURE CleanJob */

/*************************************************************************/
/* Delete JobCtrl records that are "stopped".                            */
/*************************************************************************/

PROCEDURE CleanJobCtrl:
  
  PUT UNFORMATTED "Looking for JobCtrl records to delete..." SKIP.

  FOR EACH JobCtrl EXCLUSIVE-LOCK
    WHERE JobCtrl.Operation = "Stopped":

    PUT UNFORMATTED "Deleting Record For PID " JobCtrl.PrcHostPID
                    " on Host " JobCtrl.PrcHostName 
                    SKIP.
                    
    DELETE JobCtrl.
    
  END. /* FOR EACH JobCtrl */
  
  PUT UNFORMATTED "Done looking for JobCtrl records to delete..." SKIP.  
  
END. /* PROCEDURE CleanJobCtrl */

/*************************************************************************/
/* Is there any ActionAlerts for this process we need to take care of?   */
/*************************************************************************/

PROCEDURE CheckJobCtrl:

  DEFINE BUFFER This_JobCtrl FOR JobCtrl.
  DEFINE BUFFER This_LockedJobCtrl FOR JobCtrl.
  
  /***********************************************************************/
  /* We FOR EACH in case there is "noise" in the table.                  */
  /***********************************************************************/
  
  FOR EACH This_JobCtrl NO-LOCK
  WHERE This_JobCtrl.PrcHostName = cgThisHost
    AND This_JobCtrl.PrcHostPID = cgThisPID
    AND This_JobCtrl.Operation <> "Stopped":
    
    CASE This_JobCtrl.ActionAlert:

      /**************************************************/      
      /* Kill this process.                             */
      /**************************************************/                                                 
      
      WHEN "STOP" THEN DO:
       
         PUT UNFORMATTED "Stop issued for this process" SKIP.
         
         FIND This_LockedJobCtrl EXCLUSIVE-LOCK 
         WHERE ROWID(This_LockedJobCtrl) = ROWID(This_JobCtrl)
         NO-ERROR.
         
         ASSIGN This_LockedJobCtrl.Operation = "Stopped".
                                                                                            
         QUIT.
         
      END. /* WHEN STOP */
      
    END. /* CASE This_JobCtrl.ActionAlert */
    
  END. /* FOR EACH This_JobCtrl */
  
END. /* PROCEDURE CheckJobCtrl */

/*************************************************************************/
/* Actually look for something to do!                                    */
/*************************************************************************/

PROCEDURE CheckJob:

  FIND FIRST Job NO-LOCK
  WHERE Job.State = "SUBMITTED"
    AND Job.Name = cName
  NO-ERROR.
  
  IF NOT AVAILABLE Job THEN RETURN.

  /****************************************************************/      
  /* Lock up record for writing, then unlock so monitor processes */
  /* and WatchJob can get to the record.                          */
  /****************************************************************/
  
  FIND CURRENT Job EXCLUSIVE-LOCK.
  
  ASSIGN 
  Job.State = "WIP"
  Job.PrcStartTime = TIME
  Job.PrcStartDate = TODAY
  Job.PrcHostName = cgThisHost
  Job.PrcHostPID = cgThisPID.
                          
  FIND CURRENT Job NO-LOCK.
  
  FIND JobPrg NO-LOCK
  WHERE JobPrg.Name = Job.Name
  NO-ERROR.
                                                                                       
  IF NOT AVAILABLE JobPrg THEN DO:

    /****************************************************************/      
    /* Lock up record for writing, then unlock so monitor processes */
    /* and WatchJob can get to the record.                          */
    /****************************************************************/
    
    FIND CURRENT Job EXCLUSIVE-LOCK.
  
    ASSIGN 
    Job.ErrCode = "100"
    Job.State = "ERROR".
    
    FIND CURRENT Job NO-LOCK.
    
    RETURN.
    
  END. /* IF NOT AVAILABLE JobPrg */
  
  IF SEARCH (JobPrg.PrgName) = ? THEN DO:
    

    /****************************************************************/      
    /* Lock up record for writing, then unlock so monitor processes */
    /* and WatchJob can get to the record.                          */
    /****************************************************************/
    
    FIND CURRENT Job EXCLUSIVE-LOCK.
  
    ASSIGN 
    Job.ErrCode = "101"
    Job.State = "ERROR".
    
    FIND CURRENT Job NO-LOCK.
    
    RETURN.    
    
  END. /* IF SEARCH (JobPrg.PrgName) = ? */
  
  FIND CURRENT Job EXCLUSIVE-LOCK.
  
  RUN VALUE(JobPrg.PrgName) (INPUT Job.PrgDataIn, OUTPUT Job.PrgDataOut).
                                       
  ASSIGN
  Job.PrcStopTime = TIME
  Job.PrcStopDate = TODAY
  Job.State = "DONE".

  FIND CURRENT Job NO-LOCK.
  
  RELEASE Job.

END. /* PROCEDURE CheckJob */


