/*************************************************************************/
/*  Job Server  UNIX Version                                             */
/*                                                                       */
/*  Zeno Job Server                                                      */
/*  Copyright (C) 2005  Scott Auge, Amduus Information Works, Inc.       */
/*                                                                       */
/*  This program is free software; you can redistribute it and/or modify */
/*  it under the terms of the GNU General Public License as published by */
/*  the Free Software Foundation; either version 2 of the License, or    */
/*  (at your option) any later version.                                  */
/*                                                                       */
/*  This program is distributed in the hope that it will be useful,      */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/*  GNU General Public License for more details.                         */
/*                                                                       */
/*  You should have received a copy of the GNU General Public License    */
/*  along with this program; if not, write to                            */
/*                                                                       */
/*  Free Software Foundation, Inc.                                       */
/*  59 Temple Place, Suite 330                                           */
/*  Boston, MA  02111-1307  USA                                          */
/*                                                                       */
/*************************************************************************/

DEFINE VARIABLE hjobclient AS HANDLE NO-UNDO.
DEFINE VARIABLE cJobID     AS CHARACTER NO-UNDO.
DEFINE VARIABLE cState     AS CHARACTER NO-UNDO.
DEFINE VARIABLE iIter      AS INTEGER NO-UNDO.

RUN jobclient.p PERSISTENT SET hjobclient.

/*************************************************************************/
/* Clean up house a bit to aid with monitor.p in another session.        */
/*************************************************************************/

FOR EACH Job EXCLUSIVE-LOCK:
  DELETE Job.
END.

/*************************************************************************/
/* Define the service definition via APIs.                               */
/*************************************************************************/

RUN DefineService IN hjobclient (INPUT "tstsvc1", INPUT "tstsvc1.p").
RUN DefineService IN hjobclient (INPUT "tstsvc2", INPUT "tstsvc2.p").

/*************************************************************************/
/* Test creation of error 101 (No prg for service)                       */
/*************************************************************************/

RUN DefineService IN hjobclient (INPUT "tstsvc3", INPUT "tstsvcnonexistant.p").

/*************************************************************************/
/* Put out some jobs to do.                                              */
/*************************************************************************/

RUN SubmitJob IN hjobclient (INPUT "tstsvc3", INPUT 1, INPUT "Job1", OUTPUT cJobID). /* 101 */
RUN SubmitJob IN hjobclient (INPUT "tstsvc4", INPUT 1, INPUT "Job1", OUTPUT cJobID). /* 100 */
RUN SubmitJob IN hjobclient (INPUT "tstsvc1", INPUT 1, INPUT "Job1", OUTPUT cJobID).
RUN SubmitJob IN hjobclient (INPUT "tstsvc2", INPUT 1, INPUT "Job1", OUTPUT cJobID).

/*************************************************************************/
/* tstsvc2.p has 30 second pause 20*2 seconds should be long enough for  */
/* the process to pick it up and provide the service.                    */
/*************************************************************************/

DO iIter = 1 TO 20:  
  RUN WatchJob IN hjobclient (INPUT cJobID, OUTPUT cState).
  DISPLAY cState.
  PAUSE 2.
END. /* DO iIter = 1 TO 10 */

/*************************************************************************/
/* Undefine the service definition via APIs.                             */
/* PAUSE to help with monitoring record creation and deletion.           */
/*************************************************************************/

PAUSE 40.


RUN RemoveService IN hjobclient ("tstsvc1").
RUN RemoveService IN hjobclient ("tstsvc2").
RUN RemoveService IN hjobclient ("tstsvc3").

